@@ -41,6 +41,48 @@ def test_uv_venv_pass_env(tox_project: ToxProjectCreator) -> None:
     assert "PKG_CONFIG_PATH" in pass_through
 
 
+@pytest.mark.usefixtures("clear_python_preference_env_var")
+def test_uv_venv_preference_system_by_default(tox_project: ToxProjectCreator) -> None:
+    project = tox_project({"tox.ini": "[testenv]"})
+
+    result = project.run("c", "-k", "uv_python_preference")
+    result.assert_success()
+
+    parser = ConfigParser()
+    parser.read_string(result.out)
+    got = parser["testenv:py"]["uv_python_preference"]
+
+    assert got == "system"
+
+
+def test_uv_venv_preference_override_via_env_var(
+    tox_project: ToxProjectCreator, monkeypatch: pytest.MonkeyPatch
+) -> None:
+    project = tox_project({"tox.ini": "[testenv]"})
+    monkeypatch.setenv("UV_PYTHON_PREFERENCE", "only-managed")
+
+    result = project.run("c", "-k", "uv_python_preference")
+    result.assert_success()
+
+    parser = ConfigParser()
+    parser.read_string(result.out)
+    got = parser["testenv:py"]["uv_python_preference"]
+
+    assert got == "only-managed"
+
+
+def test_uv_venv_preference_override_via_env_var_and_set_env_depends_on_py(
+    tox_project: ToxProjectCreator, monkeypatch: pytest.MonkeyPatch
+) -> None:
+    project = tox_project({"tox.ini": "[testenv]\nset_env=A={env_site_packages_dir}"})
+    monkeypatch.setenv("UV_PYTHON_PREFERENCE", "only-managed")
+
+    result = project.run("c", "-k", "set_env")
+    result.assert_success()
+
+    assert str(project.path) in result.out
+
+
 def test_uv_venv_spec(tox_project: ToxProjectCreator) -> None:
     ver = sys.version_info
     project = tox_project({"tox.ini": f"[testenv]\npackage=skip\nbase_python={ver.major}.{ver.minor}"})
@@ -339,3 +381,42 @@ def test_uv_python_set(tox_project: ToxProjectCreator, monkeypatch: pytest.Monke
     })
     result = project.run("-vv")
     result.assert_success()
+
+
+def test_uv_pip_constraints(tox_project: ToxProjectCreator) -> None:
+    project = tox_project({
+        "tox.ini": f"""
+            [testenv]
+            package=skip
+            setenv=
+                PIP_CONSTRAINTS={os.devnull}
+            commands=python --version
+            """
+    })
+    result = project.run()
+    result.assert_success()
+    assert (
+        result.out.count(
+            "Found PIP_CONSTRAINTS defined, you may want to also define UV_CONSTRAINT to match pip behavior."
+        )
+        == 1
+    ), "Warning should be found once and only once in output."
+
+
+def test_uv_pip_constraints_no(tox_project: ToxProjectCreator) -> None:
+    project = tox_project({
+        "tox.ini": f"""
+            [testenv]
+            package=skip
+            setenv=
+                PIP_CONSTRAINTS={os.devnull}
+                UV_CONSTRAINT={os.devnull}
+            commands=python --version
+            """
+    })
+    result = project.run()
+    result.assert_success()
+    assert (
+        "Found PIP_CONSTRAINTS defined, you may want to also define UV_CONSTRAINT to match pip behavior."
+        not in result.out
+    )
